import { NewsService } from '../../service/platform/news.service';
import { CVO_news, FVO_news, QVO_news, UVAO_news } from '../../interface/platform/news.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ServiceError, ErrorCode } from '../../error/service.error';
import { Controller, Inject, Get, Param, Post, Body, Del, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
@ApiTags(['新闻'])
@Controller('/news')
export class NewsController implements BaseController {
  @Inject()
  service: NewsService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_news })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const qbr = this.service.queryBuilder(qobj);
    const result = await this.service.query(qbr, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_news })
  async fetch(@Param('id') id: number) {
    const qbr = this.service.queryBuilder({ id });
    const data = await this.service.fetch(qbr);
    await this.service.fetchBrowse(data);
    const result = new FVO_news(data);
    return result;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_news })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_news(dbData);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_news })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
