import { Body, Controller, Del, Get, Inject, Param, Post, Put, Query } from '@midwayjs/core';
import { AdminService } from '../../service/system/admin.service';
import { omit, pick } from 'lodash';
import { ApiResponse, ApiTags } from '@midwayjs/swagger';
import { BaseController } from '../../frame/BaseController';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { Validate } from '@midwayjs/validate';
import { CVO_admin, FVO_admin, QVO_admin, UVAO_admin } from '../../interface/system/admin.interface';

@ApiTags(['管理用户表'])
@Controller('/admin')
export class HomeController implements BaseController {
  controllerCode = 'user_admin';
  @Inject()
  service: AdminService;
  @Get('/')
  @ApiTags('列表查询')
  @ApiResponse({ type: QVO_admin })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const qbr = this.service.queryBuilder(qobj);
    const result = await this.service.query(qbr, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_admin })
  async fetch(@Param('id') id: number) {
    const qbr = this.service.queryBuilder({ id });
    const data = await this.service.fetch(qbr);
    const result = new FVO_admin(data);
    return result;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_admin })
  async create(@Body() data: object) {
    const result = await this.service.create(data);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_admin })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    const result = await this.service.delete({ id });
    return result;
  }

  @Put('/init')
  async init() {
    await this.service.initSuperAdmin();
    return 'ok';
  }
}
