import { Body, Controller, Del, Get, Inject, Param, Post, Put, Query } from '@midwayjs/core';
import { omit, pick } from 'lodash';
import { ApiResponse, ApiTags } from '@midwayjs/swagger';
import { BaseController } from '../../frame/BaseController';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { Validate } from '@midwayjs/validate';
import { MenusService } from '../../service/system/menus.service';
import { CVO_menus, FVO_menus, QVO_menus, UVAO_menus } from '../../interface/system/menus.interface';

@ApiTags(['菜单表'])
@Controller('/menus')
export class MenusController implements BaseController {
  controllerCode = 'system_menus';
  @Inject()
  service: MenusService;
  @Get('/')
  @ApiTags('列表查询')
  @ApiResponse({ type: QVO_menus })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const qbr = this.service.queryBuilder(qobj);
    const result = await this.service.query(qbr, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_menus })
  async fetch(@Param('id') id: number) {
    const qbr = this.service.queryBuilder({ id });
    const data = await this.service.fetch(qbr);
    const result = new FVO_menus(data);
    return result;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_menus })
  async create(@Body() data: object) {
    const result = await this.service.create(data);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_menus })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    const result = await this.service.delete({ id });
    return result;
  }

  @Put('/init')
  async init() {
    await this.service.initData();
    return 'ok';
  }
}
