import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { BaseController } from '../../frame/BaseController';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { UserService } from '../../service/system/user.service';
import { CVO_user, FVO_user, QVO_user, UVAO_user } from '../../interface/system/user.interface';

@ApiTags(['平台用户'])
@Controller('/user')
export class UserController implements BaseController {
  controllerCode = 'user_user';
  @Inject()
  service: UserService;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_user })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const qbr = this.service.queryBuilder(qobj);
    const result = await this.service.query(qbr, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_user })
  async fetch(@Param('id') id: number) {
    const qbr = this.service.queryBuilder({ id });
    const data = await this.service.fetch(qbr);
    const result = new FVO_user(data);
    return result;
  }

  @Get('/detail/:id')
  @ApiResponse({ type: FVO_user })
  async detail(@Param('id') id: string) {
    const data = await this.service.detail(id);
    const result = new FVO_user(data);
    return result;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_user })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_user(dbData);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_user })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
