import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/core';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { UtilService } from '../../service/util.service';
import { BaseController } from '../../frame/BaseController';
import { omit, pick } from 'lodash';
import { ErrorCode, ServiceError } from '../../error/service.error';
import { CompanyService } from '../../service/users/company.service';
import { CVO_company, FVO_company, QVO_company, UVAO_company } from '../../interface/users/company.interface';
import { Context } from '@midwayjs/koa';
@ApiTags(['企业'])
@Controller('/company')
export class CompanyController implements BaseController {
  @Inject()
  service: CompanyService;

  @Inject()
  utilService: UtilService;

  @Inject()
  ctx: Context;

  @Get('/')
  @ApiTags('列表查询')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_company })
  async index(@Query() query: object) {
    const qobj = omit(query, ['skip', 'limit']);
    const others = pick(query, ['skip', 'limit']);
    const qbr = this.service.queryBuilder(qobj);
    const result = await this.service.query(qbr, others);
    return result;
  }

  @Get('/:id')
  @ApiTags('单查询')
  @ApiResponse({ type: FVO_company })
  async fetch(@Param('id') id: number) {
    const qbr = this.service.queryBuilder({ id });
    const data = await this.service.fetch(qbr);
    const result = new FVO_company(data);
    return result;
  }

  @Get('/list')
  async list() {
    const list = await this.service.list(this.ctx.filter);
    const data = list.data;
    const total = list.total;
    return { data, total };
  }
  @Get('/detail/:id')
  @ApiResponse({ type: FVO_company })
  async detail(@Param('id') id: string) {
    const data = await this.service.detail(id);
    return data;
  }

  @Post('/')
  @ApiTags('创建数据')
  @Validate()
  @ApiResponse({ type: CVO_company })
  async create(@Body() data: object) {
    const dbData = await this.service.create(data);
    const result = new CVO_company(dbData);
    return result;
  }

  @Post('/:id')
  @ApiTags('修改数据')
  @Validate()
  @ApiResponse({ type: UVAO_company })
  async update(@Param('id') id: number, @Body() data: object) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.update({ id }, data);
    await this.utilService.updateUserAfter(id, data, 'Company');
    return result;
  }

  @Del('/:id')
  @ApiTags('删除数据')
  @Validate()
  async delete(@Param('id') id: number) {
    if (!id) throw new ServiceError(ErrorCode.ID_NOT_FOUND);
    const result = await this.service.delete({ id });
    return result;
  }
}
